@@ -5,12 +5,12 @@
 #include "includes.h"
 #include "gdc.h"
 
-struct	GDC_FONT_T	GDC_fontc[GDC_numfonts] = { (gdFontPtr)NULL, 8,  5,
-											    (gdFontPtr)NULL, 8,  5,
-											    (gdFontPtr)NULL, 12, 6,
-											    (gdFontPtr)NULL, 13, 7,
-											    (gdFontPtr)NULL, 16, 8,
-											    (gdFontPtr)NULL, 15, 9 };
+struct	GDC_FONT_T	GDC_fontc[GDC_numfonts] = { 					   {(gdFontPtr)NULL, 8,  5},
+											   {(gdFontPtr)NULL, 8,  5},
+											   {(gdFontPtr)NULL, 12, 6},
+											   {(gdFontPtr)NULL, 13, 7},
+											   {(gdFontPtr)NULL, 16, 8},
+											   {(gdFontPtr)NULL, 15, 9 }};
 
 /* ------------------------------------------------------------------- *\ 
  * convert from enum GDC_font_size to gd fonts
