@@ -290,8 +290,8 @@ monitor_list_update(ctrls,msg, short_status)
    gmax = max;
    gcurrent = atoi(current);
    f = (gcurrent/gmax);
-   if(f>1)f=1;
-   if(f<0.0)f=0.0;
+   if(f>=1.0)f=1.0;
+   if(f<=0.0)f=0.0;
    gtk_progress_bar_update (GTK_PROGRESS_BAR(progress_bar), f);
 
    flag = 1;
