@@ -15,6 +15,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
+ * In addition, as a special exception, Renaud Deraison
+ * gives permission to link the code of this program with any
+ * version of the OpenSSL library which is distributed under a
+ * license identical to that listed in the included COPYING.OpenSSL
+ * file, and distribute linked combinations including the two.
+ * You must obey the GNU General Public License in all respects
+ * for all of the code used other than OpenSSL.  If you modify
+ * this file, you may extend this exception to your version of the
+ * file, but you are not obligated to do so.  If you do not wish to
+ * do so, delete this exception statement from your version.
  */
  
 #include <includes.h>
@@ -31,26 +41,40 @@
 #include "../comm.h"
 
 
+static void
+save_session_cb(w, ctrls)
+ GtkWidget* w;
+ struct arglist *ctrls;
+{
+ GtkWidget * wg = arg_get_value(ctrls, "SAVE_EMPTY_SESSIONS");
+ gtk_widget_set_sensitive(wg, GTK_TOGGLE_BUTTON(w) -> active);   
+}
+
 static void
 delete_session_cb(nul, clist)
  GtkWidget* nul, *clist;
 {
  GList * selection;
  char * key;
+ int data = -1;
+ int n = 0;
  
  if(!clist)return;
- gtk_clist_freeze(GTK_CLIST(clist));
+ /*gtk_clist_freeze(GTK_CLIST(clist));*/
+ if(!GTK_CLIST(clist)->rows)n++;
+ 
+
  selection = GTK_CLIST(clist)->selection;
  if(selection)
  {
-  key = gtk_clist_get_row_data(GTK_CLIST(clist), (int)selection->data);
+  data = (int)selection->data;
+  key = gtk_clist_get_row_data(GTK_CLIST(clist), data);
   comm_delete_session(key);
-  gtk_clist_remove(GTK_CLIST(clist), (int)selection->data);
+  gtk_clist_remove(GTK_CLIST(clist), data);
   /*selection = selection->next;*/
  }
- 
- gtk_clist_thaw(GTK_CLIST(clist));
- 
+ /*   gtk_clist_thaw(GTK_CLIST(clist));
+    gtk_widget_realize(clist);*/
 }
 
 
@@ -110,7 +134,7 @@ restore_session_cb(nul, clist)
    if(!strlen(expansion))strncpy(expansion, "none;", 5);
    expansion[strlen(expansion)-1]=0;
    
-   s = gtk_entry_get_text(GTK_ENTRY(arg_get_value(t, "TARGET")));
+   s = (char*)gtk_entry_get_text(GTK_ENTRY(arg_get_value(t, "TARGET")));
    hostname = target_translate(s); /* if the target is a file, then
    				      translate it */
    
@@ -128,8 +152,12 @@ restore_session_cb(nul, clist)
    * Plugins preferences
    */
   {
-   struct arglist * plugs[2] = {Plugins, Scanners};
+   struct arglist * plugs[2];
    int i;
+   
+   plugs[0] = Plugins;
+   plugs[1] = Scanners;
+   
    for(i=0;i<2;i++)
    {
     struct arglist  * plugins = plugs[i];
@@ -150,7 +178,7 @@ restore_session_cb(nul, clist)
 	  GtkWidget * entry = arg_get_value(pref->value, "ENTRY");
 	  if(entry)
 	  {
-	  value = gtk_entry_get_text(GTK_ENTRY(entry));			      
+	  value = (char*)gtk_entry_get_text(GTK_ENTRY(entry));			      
 	  arg_set_value(pref->value, "value", strlen(value), 
 	 	       estrdup(value));
 	  }
@@ -236,7 +264,7 @@ restore_session_cb(nul, clist)
   
   
  gtkw = arg_get_value(t, "PORT_RANGE");
- s = emalloc(strlen(gtk_entry_get_text(GTK_ENTRY(gtkw)))+1);
+ s = emalloc(strlen((char*)gtk_entry_get_text(GTK_ENTRY(gtkw)))+1);
  strncpy(s, gtk_entry_get_text(GTK_ENTRY(gtkw)), 
  	strlen(gtk_entry_get_text(GTK_ENTRY(gtkw))));
  
@@ -245,21 +273,33 @@ restore_session_cb(nul, clist)
  else
   arg_add_value(serv, "port_range", ARG_STRING, strlen(s), s);
          
- gtkw = arg_get_value(t, "MAX_THREADS");
+ gtkw = arg_get_value(t, "MAX_HOSTS");
  
  s = emalloc(strlen(gtk_entry_get_text(GTK_ENTRY(gtkw)))+1);
   
  strncpy(s, gtk_entry_get_text(GTK_ENTRY(gtkw)), 
   	strlen(gtk_entry_get_text(GTK_ENTRY(gtkw))));
   
- if(arg_get_value(serv, "max_threads"))
-    arg_set_value(serv, "max_threads", strlen(s), s);
+ if(arg_get_value(serv, "max_hosts"))
+    arg_set_value(serv, "max_hosts", strlen(s), s);
   else
-    arg_add_value(serv, "max_threads", ARG_STRING, strlen(s), s);   
+    arg_add_value(serv, "max_hosts", ARG_STRING, strlen(s), s);
+    
+ gtkw = arg_get_value(t, "MAX_CHECKS");
+ 
+ s = emalloc(strlen(gtk_entry_get_text(GTK_ENTRY(gtkw)))+1);
+  
+ strncpy(s, gtk_entry_get_text(GTK_ENTRY(gtkw)), 
+  	strlen(gtk_entry_get_text(GTK_ENTRY(gtkw))));
+  
+ if(arg_get_value(serv, "max_checks"))
+    arg_set_value(serv, "max_checks", strlen(s), s);
+  else
+    arg_add_value(serv, "max_checks", ARG_STRING, strlen(s), s);      
  
  
  gtkw = arg_get_value(t, "CGI_PATH");
- s = gtk_entry_get_text(GTK_ENTRY(gtkw));
+ s = (char*)gtk_entry_get_text(GTK_ENTRY(gtkw));
  s = emalloc(strlen(s)+1);
  strncpy(s, gtk_entry_get_text(GTK_ENTRY(gtkw)), 
   	strlen(gtk_entry_get_text(GTK_ENTRY(gtkw))));
@@ -313,10 +353,19 @@ prefs_dialog_target_fill_sessions(dialog, sessions)
   int i = 0;
   GtkWidget * clist = arg_get_value(dialog, "SESSIONS");
   int last;
-  char*empty[] = {"", ""};
+  char**empty;
+  
+
+  
   if(!clist)return;
- 
   if(!sessions)return;
+  
+  empty = emalloc(2*sizeof(char*));
+  empty[0] = estrdup("");
+  empty[1] = estrdup("");
+  
+
+  
   hw = harg_walk_init(sessions);
   gtk_clist_freeze(GTK_CLIST(clist));
   last = gtk_clist_append(GTK_CLIST(clist), empty);
@@ -324,16 +373,21 @@ prefs_dialog_target_fill_sessions(dialog, sessions)
   i = 0;
   while((key = (char*)harg_walk_next(hw)))
   {
-   char * data[] = {key, harg_get_string(sessions, key)};
-   int row =  gtk_clist_append(GTK_CLIST(clist), data);
+   char * data[2];
+   int row;
+   data[0] = key;
+   data[1] = harg_get_string(sessions, key);
+   row =  gtk_clist_append(GTK_CLIST(clist), data);
    gtk_clist_set_row_data(GTK_CLIST(clist), row, key);
    i++; 
   }
+#if GTK_VERSION > 10
   gtk_clist_sort(GTK_CLIST(clist));
   gtk_clist_set_column_width(GTK_CLIST(clist),
  			0,
 			gtk_clist_optimal_column_width(GTK_CLIST(clist), 0)
 			);
+#endif
   gtk_clist_thaw(GTK_CLIST(clist));
 }
 struct arglist * prefs_dialog_target(preferences)
@@ -360,7 +414,7 @@ struct arglist * prefs_dialog_target(preferences)
  arg_add_value(ctrls, "FRAME", ARG_PTR, -1, frame);
  
  
- table = gtk_table_new(6, 2, FALSE);
+ table = gtk_table_new(7, 2, FALSE);
  gtk_container_add(GTK_CONTAINER(frame), table);
  gtk_container_border_width(GTK_CONTAINER(table), 10);
  gtk_table_set_row_spacings(GTK_TABLE(table), 5);
@@ -401,16 +455,31 @@ struct arglist * prefs_dialog_target(preferences)
  arg_add_value(ctrls, "SAVE_THIS_SESSION", ARG_PTR, -1, button);
  gtk_table_attach(GTK_TABLE(table), button, 0,2, 2, 3, GTK_FILL|GTK_EXPAND,0,0,0);
  gtk_widget_show(button);
+ gtk_signal_connect(GTK_OBJECT(button),
+		     "clicked",
+		     GTK_SIGNAL_FUNC(save_session_cb),
+		     ctrls);
+		     
+		     
+ 
+ hbox = gtk_hbox_new(TRUE, 10);
+ gtk_table_attach(GTK_TABLE(table), hbox, 0,2,3,4, GTK_FILL|GTK_EXPAND,0,0,0);
+ gtk_widget_show(hbox);
+ 
+ button = gtk_check_button_new_with_label("Save empty sessions");
+ arg_add_value(ctrls, "SAVE_EMPTY_SESSIONS", ARG_PTR, -1, button);
+ gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 10);
+ gtk_widget_show(button);
  
  label = gtk_label_new("Previous sessions : ");
- gtk_table_attach(GTK_TABLE(table), label, 0, 2, 3, 4,GTK_FILL|GTK_EXPAND,0,0,0);
+ gtk_table_attach(GTK_TABLE(table), label, 0, 2, 4, 5,GTK_FILL|GTK_EXPAND,0,0,0);
  gtk_widget_show(label);
  
  scrolled = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
 				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 				 
- gtk_table_attach(GTK_TABLE(table), scrolled, 0, 2, 4, 5,GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND,0,0);				 
+ gtk_table_attach(GTK_TABLE(table), scrolled, 0, 2, 5, 6,GTK_FILL|GTK_EXPAND,GTK_FILL|GTK_EXPAND,0,0);				 
  gtk_widget_show(scrolled);
  clist = gtk_clist_new_with_titles(2, titles);
 #if GTK_VERSION < 11
@@ -421,11 +490,13 @@ struct arglist * prefs_dialog_target(preferences)
  gtk_widget_show(clist);
 
  gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_SINGLE);
+#if GTK_VERSION > 10
  gtk_clist_set_reorderable(GTK_CLIST(clist), TRUE);
+#endif
  arg_add_value(ctrls, "SESSIONS", ARG_PTR, -1, clist);
  
  hbox = gtk_hbox_new(TRUE, 5);
- gtk_table_attach(GTK_TABLE(table), hbox, 0, 2, 5, 6, GTK_FILL|GTK_EXPAND,0,0,0);
+ gtk_table_attach(GTK_TABLE(table), hbox, 0, 2, 6, 7, GTK_FILL|GTK_EXPAND,0,0,0);
  gtk_widget_show(hbox);
  
  button = gtk_button_new_with_label("Restore session");
