@@ -20,5 +20,6 @@
 #define _NESSUSC_PREFS_TARGET_H
 
 struct arglist * prefs_dialog_target(struct arglist *);
-
+void prefs_dialog_target_free_list(struct arglist*);
+void prefs_dialog_target_fill_sessions(struct arglist*, harglst*);
 #endif
