@@ -751,9 +751,6 @@ create_label(be, list, obj)
  GtkWidget *widget = gtk_list_item_new();
  GtkWidget * box = gtk_hbox_new(FALSE, 3);
  GtkWidget * hostname;
- GdkColormap *colormap;
- GdkPixmap *gdkpixmap;
- GdkBitmap *mask;
  GtkWidget *pixmap;
  char * sort_key = gtk_object_get_data(GTK_OBJECT(list), "sort_key");
  char ** cat = NULL;
@@ -771,29 +768,16 @@ create_label(be, list, obj)
  
  if(cat)
  {
- colormap = gtk_widget_get_colormap (widget);
- gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL, colormap, &mask,
-                                                     NULL, (char**)cat);
-						     
- pixmap = gtk_pixmap_new (gdkpixmap, mask);
- gdk_pixmap_unref (gdkpixmap);
- gdk_bitmap_unref (mask);
- 
- gtk_box_pack_start(GTK_BOX(box), pixmap, FALSE, FALSE, 5);
- gtk_widget_show(pixmap);
+   pixmap = make_pixmap(list, NULL, cat);
+   gtk_box_pack_start(GTK_BOX(box), pixmap, FALSE, FALSE, 5);
+   gtk_widget_show(pixmap);
  }
  
  if((level = select_severity_pixmap(severity)))
  {
- colormap = gtk_widget_get_colormap (widget);
- gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL, colormap, &mask,
-                                                     NULL, level);
- pixmap = gtk_pixmap_new (gdkpixmap, mask);
- gdk_pixmap_unref (gdkpixmap);
- gdk_bitmap_unref (mask);
- 
- gtk_box_pack_start(GTK_BOX(box), pixmap, FALSE, FALSE, 5);
- gtk_widget_show(pixmap);
+   pixmap = make_pixmap(list, NULL, level);
+   gtk_box_pack_start(GTK_BOX(box), pixmap, FALSE, FALSE, 5);
+   gtk_widget_show(pixmap);
  }
  
  t = strchr(name, '\r');
