@@ -1,5 +1,5 @@
 /* Nessus
- * Copyright (C) 1998, 1999, 2000 Renaud Deraison
+ * Copyright (C) 1998 - 2004 Renaud Deraison
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -15,16 +15,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * In addition, as a special exception, Renaud Deraison
- * gives permission to link the code of this program with any
- * version of the OpenSSL library which is distributed under a
- * license identical to that listed in the included COPYING.OpenSSL
- * file, and distribute linked combinations including the two.
- * You must obey the GNU General Public License in all respects
- * for all of the code used other than OpenSSL.  If you modify
- * this file, you may extend this exception to your version of the
- * file, but you are not obligated to do so.  If you do not wish to
- * do so, delete this exception statement from your version.
  *
  */
  
@@ -101,7 +91,7 @@ void _detached_send_mail(globals)
   /*
    * Close all open files
    */
-  for(i=0;i<1024;i++)close(i);
+  for(i=0;i<getdtablesize();i++)close(i);
   
   /*
    * Our email file becomes stdin
@@ -312,6 +302,7 @@ file needed for detached scan (%s) - aborting <|> SERVER\n", strerror(errno));
    fprintf(fl, "From: Nessus Daemon <root@%s>\n", hostname);
    fprintf(fl, "To: Nessus User <%s>\n", email);
    fprintf(fl, "Subject: Nessus Scan (%s)\n\n", today);
+   fflush(fl);
    efree(&today); 
    efree(&hostname);
    stat(tmpname, &st);
@@ -417,7 +408,7 @@ detached_new_session(globals, target)
  char * target;
 {
  char * fname = detached_fname(globals);
- int f = open(fname, O_CREAT|O_WRONLY);
+ int f = open(fname, O_CREAT|O_WRONLY|O_TRUNC);
  if(f >= 0)
  {
   write(f, target, strlen(target));
@@ -502,7 +493,7 @@ detached_delete_session(globals, index)
  {
   close(f);
   if(index != getpid())
-   terminate_process(index);
+   kill(index, SIGTERM);
   unlink(file);
  }
  efree(&file);
