@@ -51,7 +51,11 @@ log_init(filename)
 	}
   else
     {
-      int fd = open(filename, O_WRONLY|O_CREAT|O_APPEND, 0644);
+      int fd = open(filename, O_WRONLY|O_CREAT|O_APPEND
+#ifdef O_LARGEFILE
+	| O_LARGEFILE
+#endif
+	, 0644);
       if(fd < 0)
       {
        perror("log_init():open ");
