@@ -14,7 +14,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: nessusd.c,v 1.236 2004/10/27 20:38:57 renaud Exp $
+ * $Id: nessusd.c,v 1.236.2.2 2005/03/21 22:55:19 renaud Exp $
  */
 
 #include <includes.h>
@@ -95,6 +95,7 @@ static void server_thread (struct arglist *);
 
 
 
+
 static struct in_addr * convert_ip_addresses(char * ips)
 {
  char * t;
@@ -232,7 +233,11 @@ start_daemon_mode
   fflush(stdout);
   fflush(stderr);
 
-    if ((fd = open (s, O_WRONLY|O_CREAT|O_APPEND, 0600)) < 0) {
+    if ((fd = open (s, O_WRONLY|O_CREAT|O_APPEND
+#ifdef O_LARGEFILE
+	| O_LARGEFILE
+#endif
+	, 0600)) < 0) {
       log_write ("Cannot create a new dumpfile %s (%s)-- aborting\n",
 		 s, strerror (errno));
       exit (2);
@@ -761,7 +766,7 @@ main_loop()
       
       if(restart != 0) restart_nessusd(); 
 
-      wait_for_children();
+      wait_for_children1();
       /* prevent from an io table overflow attack against nessus */
       if (asciiaddr != 0) {
 	time_t now = time (0);
