@@ -53,13 +53,13 @@ void plugin_set_running_state(plugins_scheduler_t shed, struct scheduler_plugin
 static unsigned int mkhash(char * name)
 {
   int i;
-  int h = 0;
+  unsigned int h = 0;
   
   for(i=0; name[i] != '\0' ;i++)
-   h = ((h * 128) + name[i]) % HASH_MAX;
+   h += name[i]; 
    
  
-  return h;
+  return h % HASH_MAX;
 }
 
 /*------------------------------------------------------------------------------*/
@@ -253,9 +253,11 @@ static int hash_add(struct hash * h, char * name, struct scheduler_plugin * plug
  int num_deps = 0;
  
  l->plugin = plugin;
+ l->plugin->parent_hash = l;
  l->name   = name;
  l->next = h[idx].next;
  h[idx].next = l;
+ l->dependencies_ptr = NULL;
  
  if( deps == NULL )
   l->dependencies = NULL;
@@ -306,6 +308,7 @@ static int hash_add(struct hash * h, char * name, struct scheduler_plugin * plug
 
 
 
+
 static struct hash * _hash_get(struct hash * h, char * name)
 {
  unsigned int idx = mkhash(name);
@@ -320,38 +323,37 @@ static struct hash * _hash_get(struct hash * h, char * name)
  return NULL;
 }
 
-static struct scheduler_plugin * hash_get(struct hash * h, char * name)
-{
- struct hash * l = _hash_get(h, name);
- if(  l == NULL )
-  return NULL;
- else
-  return l->plugin;
-}
 
-static char ** hash_get_ports(struct hash * h, char * name)
-{
- struct hash * l = _hash_get(h, name);
- if( l == NULL)
-  return NULL;
- else
-  return l->ports;
-}
-
-
-static char ** hash_get_deps(struct hash * h, char * name)
+static struct hash ** hash_get_deps_ptr(struct hash * h, char * name)
 {
  struct hash * l = _hash_get(h, name);
  
  if( l == NULL )
   return NULL;
  
- if( l->dependencies == NULL )
+ if( l->dependencies_ptr == NULL )
   return NULL;
  
- return l->dependencies;
+ return l->dependencies_ptr;
 }
 
+static void hash_fill_deps(struct hash * h, struct hash * l )
+{
+ int i, j = 0;
+ if ( l->num_deps != 0 )
+ {
+  l->dependencies_ptr = emalloc((1 + l->num_deps) * sizeof(struct hash*));
+  for ( i = 0 ; l->dependencies[i] ; i ++ )
+	{
+	struct hash * d = _hash_get(h, l->dependencies[i]);
+	if ( d != NULL )
+	 l->dependencies_ptr[j++] = d;
+        else
+          log_write("%s depends on %s which could not be found\n", l->plugin->arglist->name, l->dependencies[i]);
+	}
+   l->dependencies_ptr[j] = NULL;
+  }
+}
 
 /*----------------------------------------------------------------------*/
 
@@ -378,7 +380,7 @@ struct plist * pl_get(struct plist * list, char * name)
 
 void scheduler_mark_running_ports(plugins_scheduler_t sched, struct scheduler_plugin * plugin)
 {
- char ** ports = hash_get_ports(sched->hash, plugin->arglist->name);
+ char ** ports = plugin->parent_hash->ports;
  int i;
  
  if( ports == NULL )
@@ -409,9 +411,8 @@ void scheduler_rm_running_ports(plugins_scheduler_t sched, struct scheduler_plug
  char ** ports; 
  int i;
 
-
  
- ports = hash_get_ports(sched->hash, plugin->arglist->name);
+ ports = plugin->parent_hash->ports;
 
  
  if( ports == NULL )
@@ -490,19 +491,18 @@ void scheduler_plugin_best_score(plugins_scheduler_t sched, int *bscore, struct
 
 
 
-struct scheduler_plugin * plugin_next_unrun_dependencie(plugins_scheduler_t sched, char ** dependencies, int already_in_dependencie)
+struct scheduler_plugin * plugin_next_unrun_dependencie(plugins_scheduler_t sched, struct hash ** dependencies_ptr, int already_in_dependencie)
 {
- struct hash * h = sched->hash;
  int flag = 0;
  int counter = 0;
  int i;
  
- if(dependencies == NULL)
+ if(dependencies_ptr == NULL)
   return NULL;
   
- for(i=0;dependencies[i] != NULL;i++)
+ for(i=0;dependencies_ptr[i] != NULL;i++)
   {
-   struct scheduler_plugin * plugin = hash_get(h, dependencies[i]);
+   struct scheduler_plugin * plugin = dependencies_ptr[i]->plugin;
    if(plugin != NULL)
    {
     int state = plugin_get_running_state(plugin);
@@ -510,14 +510,14 @@ struct scheduler_plugin * plugin_next_unrun_dependencie(plugins_scheduler_t sche
     {
      case PLUGIN_STATUS_UNRUN :
      	{
-	char ** deps = hash_get_deps(h, plugin->arglist->name);
+	struct hash ** deps_ptr = dependencies_ptr[i]->dependencies_ptr;
 	struct scheduler_plugin * ret;
 	counter ++;
-	if(deps == NULL)
+	if(deps_ptr == NULL)
 	  return plugin;	
 	else
 	 {
-	 ret = plugin_next_unrun_dependencie(sched, deps, 1);
+	 ret = plugin_next_unrun_dependencie(sched, deps_ptr, 1);
 	 if(ret == NULL)
 		return plugin;
 	 else 
@@ -538,7 +538,6 @@ struct scheduler_plugin * plugin_next_unrun_dependencie(plugins_scheduler_t sche
     }
    }
   }
-  else fprintf(stderr, "%s could not be found\n", dependencies[i]);
  }
   
   if(flag == 0)
@@ -554,11 +553,11 @@ struct scheduler_plugin * plugin_next_unrun_dependencie(plugins_scheduler_t sche
  */
 static void enable_plugin_and_dependencies(plugins_scheduler_t shed, struct arglist * plugin, char * name, int silent)
 {
- char ** deps;
+ struct hash ** deps_ptr;
  int i;
  int status;
  
- deps = hash_get_deps(shed->hash, name);
+ deps_ptr = hash_get_deps_ptr(shed->hash, name);
 
  status = plug_get_launch(plugin);
  if ( status == LAUNCH_DISABLED )
@@ -569,16 +568,13 @@ static void enable_plugin_and_dependencies(plugins_scheduler_t shed, struct argl
    plug_set_launch(plugin, LAUNCH_SILENT);
  }
  
- if(deps != NULL)
+ if(deps_ptr != NULL)
  {
-   for(i=0;deps[i] != NULL;i++)
+   for(i=0;deps_ptr[i] != NULL;i++)
     {
      struct scheduler_plugin * p;
-     p = hash_get(shed->hash, deps[i]);
-     if( p != NULL )
-       enable_plugin_and_dependencies(shed, p->arglist->value, p->arglist->name, silent);
-     else
-       log_write("'%s' depends on '%s' which could not be found\n", name, deps[i]);
+     p = deps_ptr[i]->plugin;
+     enable_plugin_and_dependencies(shed, p->arglist->value, p->arglist->name, silent);
     }
  }
 }
@@ -590,6 +586,7 @@ plugins_scheduler_t plugins_scheduler_init(struct arglist * plugins, int autoloa
  plugins_scheduler_t ret = emalloc(sizeof(*ret));
  struct arglist * arg;
  int i;
+ struct hash * l;
  
  
  
@@ -636,6 +633,15 @@ plugins_scheduler_t plugins_scheduler_init(struct arglist * plugins, int autoloa
   }
  
  
+ for ( i = 0 ; i < HASH_MAX ; i ++ )
+ {
+  l = &ret->hash[i];
+  while ( l != NULL ){
+	 hash_fill_deps(ret->hash, l);
+	 l = l->next;
+	}
+ }
+ 
  if(autoload != 0)
  {
  arg = plugins;
@@ -719,11 +725,11 @@ struct scheduler_plugin * plugins_scheduler_next(plugins_scheduler_t h)
   {
    case PLUGIN_STATUS_UNRUN:
     {
-    char ** deps = hash_get_deps(h->hash, l->plugin->arglist->name);
+    struct hash ** deps_ptr = l->plugin->parent_hash->dependencies_ptr;
     
-    if(deps != NULL)
+    if(deps_ptr != NULL)
     {
-     struct scheduler_plugin * p = plugin_next_unrun_dependencie(h, deps, 0);
+     struct scheduler_plugin * p = plugin_next_unrun_dependencie(h, deps_ptr, 0);
      
      switch((int)p)
      {
@@ -833,3 +839,6 @@ void plugins_scheduler_free(plugins_scheduler_t sched)
  	list_destroy(sched->list[i]);	
  efree(&sched);
 }
+
+
+
