@@ -493,3 +493,14 @@ int set_linger(soc, linger)
  l.l_linger = linger;
  return setsockopt(soc, SOL_SOCKET, SO_LINGER, (void*)&l, sizeof(l));
 }
+
+
+void wait_for_children1()
+{
+ int e, n = 0;
+ do {
+ errno = 0;
+ e = waitpid(-1, NULL, WNOHANG);
+ n++;
+ } while ( (e > 0 || errno == EINTR) && n < 20 );
+}
